@@ -1,25 +1,28 @@
 package Beam::Emitter;
-our $VERSION = '1.003';
+our $VERSION = '1.004';
 # ABSTRACT: Role for event emitting classes
 
 use strict;
 use warnings;
 
-use Moo::Role;
 use Types::Standard qw(:all);
 use Scalar::Util qw( weaken refaddr );
 use Carp qw( croak );
 use Beam::Event;
+use Beam::Listener;
+use Moo::Role; # Put this last to ensure proper, automatic cleanup
+
 
 # The event listeners on this object, a hashref of arrayrefs of
-# EVENT_NAME => [ CALLBACK, ... ]
+# EVENT_NAME => [ Beam::Listener object, ... ]
+
 has _listeners => (
     is      => 'ro',
     isa     => HashRef,
     default => sub { {} },
 );
 
-#pod =method subscribe ( event_name, subref )
+#pod =method subscribe ( event_name, subref, [ %args ] )
 #pod
 #pod Subscribe to an event from this object. C<event_name> is the name of the event.
 #pod C<subref> is a subroutine reference that will get either a L<Beam::Event> object
@@ -61,11 +64,45 @@ has _listeners => (
 #pod sub. Forgetting to do that will result in a leak. The returned unsubscribe coderef
 #pod does not have this issue.
 #pod
+#pod By default, the emitter only stores the subroutine reference in an
+#pod object of class L<Beam::Listener>.  If more information should be
+#pod stored, create a custom subclass of L<Beam::Listener> and use C<%args>
+#pod to specify the class name and any attributes to be passed to its
+#pod constructor:
+#pod
+#pod   {
+#pod     package MyListener;
+#pod     extends 'Beam::Listener';
+#pod
+#pod     # add metadata with subscription time
+#pod     has sub_time => is ( 'ro',
+#pod 			  init_arg => undef,
+#pod 			  default => sub { time() },
+#pod     );
+#pod   }
+#pod
+#pod   # My::Emitter consumes the Beam::Emitter role
+#pod   my $emitter = My::Emitter->new;
+#pod   $emitter->on( "foo",
+#pod     sub { print "Foo happened!\n"; },
+#pod    class => MyListener
+#pod   );
+#pod
+#pod The L</listeners> method can be used to examine the subscribed listeners.
+#pod
+#pod
 #pod =cut
 
 sub subscribe {
-    my ( $self, $name, $sub ) = @_;
-    push @{ $self->_listeners->{$name} }, $sub;
+    my ( $self, $name, $sub, %args ) = @_;
+
+    my $class = delete $args{ class } || "Beam::Listener";
+    croak( "listener object must descend from Beam::Listener" )
+      unless $class->isa( 'Beam::Listener' );
+
+    my $listener = $class->new( %args, callback => $sub );
+
+    push @{ $self->_listeners->{$name} }, $listener;
     weaken $self;
     weaken $sub;
     return sub {
@@ -76,11 +113,12 @@ sub subscribe {
 
 #pod =method on ( event_name, subref )
 #pod
-#pod Alias for L</subscribe>.
+#pod An alias for L</subscribe>. B<NOTE>: Do not use this alias for method
+#pod modifiers! If you want to override behavior, override C<subscribe>.
 #pod
 #pod =cut
 
-*on = \&subscribe;
+sub on { shift->subscribe( @_ ) }
 
 #pod =method unsubscribe ( event_name [, subref ] )
 #pod
@@ -98,7 +136,7 @@ sub unsubscribe {
     else {
         my $listeners = $self->_listeners->{$name};
         my $idx = 0;
-        $idx++ until $idx > $#{$listeners} or refaddr $listeners->[$idx] eq refaddr $sub;
+        $idx++ until $idx > $#{$listeners} or refaddr $listeners->[$idx]->callback eq refaddr $sub;
         if ( $idx > $#{$listeners} ) {
             croak "Could not find sub in listeners";
         }
@@ -109,11 +147,12 @@ sub unsubscribe {
 
 #pod =method un ( event_name [, subref ] )
 #pod
-#pod An alias for L</unsubscribe>
+#pod An alias for L</unsubscribe>. B<NOTE>: Do not use this alias for method
+#pod modifiers! If you want to override behavior, override C<unsubscribe>.
 #pod
 #pod =cut
 
-*un = \&unsubscribe;
+sub un { shift->unsubscribe( @_ ) }
 
 #pod =method emit ( name, event_args )
 #pod
@@ -130,8 +169,8 @@ sub emit {
     return unless exists $self->_listeners->{$name};
 
     my $class = delete $args{ class } || "Beam::Event";
-    $args{ emitter  } = $self;
-    $args{ name     } = $name;
+    $args{ emitter  } ||= $self;
+    $args{ name     } ||= $name;
     my $event = $class->new( %args );
 
     # don't use $self->_listeners->{$name} directly, as callbacks may unsubscribe
@@ -139,7 +178,7 @@ sub emit {
     my @listeners = @{ $self->_listeners->{$name} };
 
     for my $listener ( @listeners  ) {
-        $listener->( $event );
+        $listener->callback->( $event );
         last if $event->is_stopped;
     }
     return $event;
@@ -165,11 +204,27 @@ sub emit_args {
     my @listeners = @{ $self->_listeners->{$name} };
 
     for my $listener ( @listeners ) {
-        $listener->( @args );
+        $listener->callback->( @args );
     }
     return;
 }
 
+#pod =method listeners ( event_name )
+#pod
+#pod Returns a list containing the listeners which have subscribed to the
+#pod specified event from this emitter.  The list elements are either
+#pod instances of L<Beam::Listener> or of custom classes specified in calls
+#pod to L</subscribe>.
+#pod
+#pod =cut
+
+sub listeners {
+
+    my ( $self, $name ) = @_;
+
+    return @{ $self->_listeners->{$name} || [] };
+}
+
 1;
 
 __END__
@@ -182,7 +237,7 @@ Beam::Emitter - Role for event emitting classes
 
 =head1 VERSION
 
-version 1.003
+version 1.004
 
 =head1 SYNOPSIS
 
@@ -343,7 +398,7 @@ someone before it does.
 
 =head1 METHODS
 
-=head2 subscribe ( event_name, subref )
+=head2 subscribe ( event_name, subref, [ %args ] )
 
 Subscribe to an event from this object. C<event_name> is the name of the event.
 C<subref> is a subroutine reference that will get either a L<Beam::Event> object
@@ -385,9 +440,36 @@ The way to fix this second example is to explicitly C<undef $cb> inside the call
 sub. Forgetting to do that will result in a leak. The returned unsubscribe coderef
 does not have this issue.
 
+By default, the emitter only stores the subroutine reference in an
+object of class L<Beam::Listener>.  If more information should be
+stored, create a custom subclass of L<Beam::Listener> and use C<%args>
+to specify the class name and any attributes to be passed to its
+constructor:
+
+  {
+    package MyListener;
+    extends 'Beam::Listener';
+
+    # add metadata with subscription time
+    has sub_time => is ( 'ro',
+			  init_arg => undef,
+			  default => sub { time() },
+    );
+  }
+
+  # My::Emitter consumes the Beam::Emitter role
+  my $emitter = My::Emitter->new;
+  $emitter->on( "foo",
+    sub { print "Foo happened!\n"; },
+   class => MyListener
+  );
+
+The L</listeners> method can be used to examine the subscribed listeners.
+
 =head2 on ( event_name, subref )
 
-Alias for L</subscribe>.
+An alias for L</subscribe>. B<NOTE>: Do not use this alias for method
+modifiers! If you want to override behavior, override C<subscribe>.
 
 =head2 unsubscribe ( event_name [, subref ] )
 
@@ -397,7 +479,8 @@ all listeners for this event.
 
 =head2 un ( event_name [, subref ] )
 
-An alias for L</unsubscribe>
+An alias for L</unsubscribe>. B<NOTE>: Do not use this alias for method
+modifiers! If you want to override behavior, override C<unsubscribe>.
 
 =head2 emit ( name, event_args )
 
@@ -414,12 +497,24 @@ directly to each subscribed callback.
 Use this if you want to avoid using L<Beam::Event>, though you miss out on the control
 features like L<stop|Beam::Event/stop> and L<stop default|Beam::Event/stop_default>.
 
+=head2 listeners ( event_name )
+
+Returns a list containing the listeners which have subscribed to the
+specified event from this emitter.  The list elements are either
+instances of L<Beam::Listener> or of custom classes specified in calls
+to L</subscribe>.
+
 =head1 SEE ALSO
 
 =over 4
 
 =item L<Beam::Event>
 
+=item L<Beam::Emitter::Cookbook>
+
+This document contains some useful patterns for your event emitters and
+listeners.
+
 =item L<http://perladvent.org/2013/2013-12-16.html>
 
 Coordinating Christmas Dinner with Beam::Emitter by Yanick Champoux.
